package it.danieleverducci.nextcloudmaps.fragments;

import static it.danieleverducci.nextcloudmaps.activity.main.GeofavoriteAdapter.SORT_BY_CREATED;
import static it.danieleverducci.nextcloudmaps.activity.main.GeofavoriteAdapter.SORT_BY_TITLE;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.fragment.app.Fragment;
import androidx.preference.PreferenceManager;
import androidx.recyclerview.widget.StaggeredGridLayoutManager;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import java.util.List;

import it.danieleverducci.nextcloudmaps.R;
import it.danieleverducci.nextcloudmaps.activity.main.GeofavoriteActivity;
import it.danieleverducci.nextcloudmaps.activity.main.GeofavoriteAdapter;
import it.danieleverducci.nextcloudmaps.activity.main.MainActivity;
import it.danieleverducci.nextcloudmaps.activity.main.SortingOrderDialogFragment;
import it.danieleverducci.nextcloudmaps.databinding.FragmentGeofavoriteListBinding;
import it.danieleverducci.nextcloudmaps.model.Geofavorite;

/**
 * Geofavorites list
 */
public class GeofavoriteListFragment extends Fragment implements GeofavoriteListConsumer, SortingOrderDialogFragment.OnSortingOrderListener {
    private FragmentGeofavoriteListBinding binding;
    private GeofavoriteAdapter geofavoriteAdapter;
    private SharedPreferences preferences;

    public GeofavoriteListFragment() {
        // Required empty public constructor
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        if (binding == null) {
            binding = FragmentGeofavoriteListBinding.inflate(
                LayoutInflater.from(container.getContext()),
                container,
                false
            );
        }

        preferences = PreferenceManager.getDefaultSharedPreferences(getContext());

        StaggeredGridLayoutManager layoutManager = new StaggeredGridLayoutManager(1, StaggeredGridLayoutManager.VERTICAL);
        binding.recyclerView.setLayoutManager(layoutManager);

        int sortRule = preferences.getInt(getString(R.string.setting_sort_by), SORT_BY_CREATED);
        GeofavoriteAdapter.ItemClickListener rvItemClickListener = new GeofavoriteAdapter.ItemClickListener() {
            @Override
            public void onItemClick(Geofavorite geofavorite) {
                Intent i = new Intent();
                i.setAction(Intent.ACTION_VIEW);
                i.setData(geofavorite.getGeoUri());
                startActivity(i);
            }

            @Override
            public void onItemShareClick(Geofavorite item) {
                Intent i = new Intent();
                i.setAction(Intent.ACTION_SEND);
                i.setType("text/plain");
                String shareMessage = getString(R.string.share_message)
                        .replace("{lat}", ""+item.getLat())
                        .replace("{lng}", ""+item.getLng());
                i.putExtra(Intent.EXTRA_TEXT, shareMessage );
                startActivity(Intent.createChooser(i, getString(R.string.share_via)));
            }

            @Override
            public void onItemDetailsClick(Geofavorite item) {
                if (getActivity() instanceof GeofavoriteActivity)
                    ((GeofavoriteActivity)getActivity()).showGeofavoriteDetailActivity(item);
                else
                    throw new IllegalStateException("Expected activity implementing GeofavoriteAcivity");
            }

            @Override
            public void onItemDeleteClick(Geofavorite item) {
                if (getActivity() instanceof GeofavoriteActivity)
                    ((GeofavoriteActivity)getActivity()).showGeofavoriteDeteleDialog(item);
                else
                    throw new IllegalStateException("Expected activity implementing GeofavoriteAcivity");
            }
        };
        geofavoriteAdapter = new GeofavoriteAdapter(getContext(), rvItemClickListener);
        binding.recyclerView.setAdapter(geofavoriteAdapter);
        geofavoriteAdapter.setSortRule(sortRule);

        binding.swipeRefresh.setOnRefreshListener(() ->
                ((GeofavoriteActivity)getActivity()).updateGeofavorites());

        return binding.root;
    }


    @Override
    public void onGeofavoritesUpdated(List<Geofavorite> geofavorites) {
        if (geofavoriteAdapter != null)
            geofavoriteAdapter.setGeofavoriteList(geofavorites);
    }

    @Override
    public void onGeofavoriteRemoved(Geofavorite geofavorite) {
        // Update list
        geofavoriteAdapter.removeById(geofavorite.getId());
    }

    @Override
    public void onGeofavoriteAdded(Geofavorite geofavorite) {

    }

    @Override
    public void onGeofavoriteLoading(boolean isLoading) {
        binding.swipeRefresh.setRefreshing(isLoading);
    }


    @Override
    public void onSortingOrderChosen(int sortSelection) {
        geofavoriteAdapter.setSortRule(sortSelection);
        updateSortingIcon(sortSelection);

        preferences.edit().putInt(getString(R.string.setting_sort_by), sortSelection).apply();
    }

    public void updateSortingIcon(int sortSelection) {
        // TODO: Gestire il bottone come bottone generico da aggiungere all'activity
        AppCompatImageView sortButton = findViewById(R.id.sort_mode);
        switch (sortSelection) {
            case SORT_BY_TITLE:
                sortButton.setImageResource(R.drawable.ic_alphabetical_asc);
                break;
            case SORT_BY_CREATED:
                sortButton.setImageResource(R.drawable.ic_modification_asc);
                break;
        }
    }
}